import vueReady from "./vue-ready";
import domReady from "./dom-ready";
import loadScript from "./load-script";
import { initSharing, registerShareModule, loadWmfModule } from "./wmf";
import detectLowercase from "./detect-lowercase";
import strToJsVar from "./str-to-js-var";

window.$HFC_CDN_URL = window.$HFC_CDN_URL || "https://cdn.jsdelivr.net/npm";
window.$HFC_WMF_CONTAINERS = {};
const autoLowercase = detectLowercase();

initSharing()
  .then(vueReady)
  .then(domReady)
  .then(() => {
    registerShareModule("vue", window.$HFC_VUE.version, window.$HFC_VUE);
    document.querySelectorAll("[hfc]").forEach(function (elem) {
      mount(elem as HTMLTemplateElement);
    });
  })
  .catch((err: any) => {
    console.warn("faild to init hyper function component loader");
    console.error(err);
  });

function mount(elem: HTMLTemplateElement) {
  Promise.all([
    import(/* webpackMode: "eager" */ "hfc-to-vue"),
    import("vue"),
  ]).then(([hfcToVue, Vue]) => {
    const {
      compile,
      createApp,
      onMounted,
      onUnmounted,
      defineComponent,
      defineAsyncComponent,
    } = Vue;

    const template = elem.innerHTML;
    let container;
    if (elem.getAttribute("hfc")) {
      container = document.querySelector(elem.getAttribute("hfc")!);
    }

    if (!container) {
      container = document.createElement("div");
      elem.insertAdjacentElement("afterend", container);
    }

    container.setAttribute("hfc", "");

    let data: any;
    const dataJsonStr = elem.getAttribute("data-json");
    if (dataJsonStr) {
      try {
        data = JSON.parse(decodeURIComponent(dataJsonStr));
      } catch (error) {}
    }

    if (!data) {
      const dataStr = elem.getAttribute(":data");
      if (dataStr) data = strToJsVar(dataStr);
    }

    data = data || {};

    let components: Record<string, any> = {
      MountEvent: defineComponent({
        emits: ["mounted", "unmounted"],
        setup({}, { slots, emit }) {
          onMounted(() => emit("mounted"));
          onUnmounted(() => emit("unmounted"));
          return () => slots.default && slots.default();
        },
      }),
    };

    const componentStr = elem.getAttribute("import");
    if (componentStr) {
      container.setAttribute("import", componentStr);
      componentStr
        .trim()
        .split(";")
        .forEach((item) => {
          const [alias, link] = item.trim().split(" ");
          if (!link || link.indexOf("@hyper.fun/") === -1) return;

          const [version, name] = link.split("/").reverse();
          const target = name + "@" + version;

          components[alias] = defineAsyncComponent({
            loader: () => {
              let cdnUrl = window.$HFC_CDN_URL;
              const rewrite = (window as any)[`$HFC_CDN_REWRITE_${target}`];
              if (rewrite) cdnUrl = rewrite;
              const componentUrl = `${cdnUrl}/@hyper.fun/${target}/wmf/hfc-entry.js`;

              return loadScript(componentUrl)
                .then(() => {
                  return loadWmfModule(name);
                })
                .then((comp: any) => {
                  return hfcToVue.default(comp.default, autoLowercase);
                })
                .catch((err) => {
                  console.warn(`faild to load component: ${alias} ${link} `);
                  return defineComponent({});
                });
            },
          });
        });
    }

    const methods: Record<string, any> = {};
    const methodStr = elem.getAttribute(":methods");
    if (methodStr) {
      const methodArr = strToJsVar(methodStr);
      if (Array.isArray(methodArr)) {
        methodArr.forEach((method) => {
          if (window[method]) methods[method] = window[method];
        });
      }
    }

    const app = createApp({
      data: () => data,
      methods,
      render: compile(template, {
        runtimeGlobalName: "$HFC_VUE",
      }),
    });

    Object.keys(components).forEach((name) => {
      app.component(
        autoLowercase ? name.toLowerCase() : name,
        components[name]
      );
    });

    app.mount(container);
    elem.remove();
  });
}
